@@ -1,6 +1,7 @@
 """
 Main File to Connect all the Scratch API and the Scratch DB
 """
+
 import requests
 import json
 import re
@@ -12,8 +13,10 @@ from scratchconnect import Project
 from scratchconnect import Studio
 from scratchconnect import User
 from scratchconnect import Forum
+from scratchconnect.scOnlineIDE import _change_request_url
 from scratchconnect.scScratchTerminal import _terminal
 from scratchconnect.scChart import _chart
+from scratchconnect.scImage import Image
 
 _website = "scratch.mit.edu"
 _login = f"https://{_website}/login/"
@@ -21,7 +24,7 @@ _api = f"api.{_website}"
 
 
 class ScratchConnect(UserCommon):
-    def __init__(self, username=None, password=None, cookie=None, auto_cookie_login=False):
+    def __init__(self, username=None, password=None, cookie=None, auto_cookie_login=False, online_ide_cookie=None):
         """
         Class to make a connection to Scratch
         :param username: The username of a Scratch Profile
@@ -30,7 +33,10 @@ class ScratchConnect(UserCommon):
         self.username = username
         self.password = password
         self.cookie = cookie
+        self.session = None
+        self._online_ide = False
         self.auto_cookie_login = auto_cookie_login
+        self.online_ide_cookie = online_ide_cookie
 
         if self.username is not None and self.password is not None:
             self._login(cookie=False, auto_cookie_login=self.auto_cookie_login)
@@ -38,6 +44,11 @@ class ScratchConnect(UserCommon):
         elif self.cookie is not None:
             self._login(cookie=True, auto_cookie_login=self.auto_cookie_login)
             self._logged_in = True
+        elif self.online_ide_cookie is not None:
+            _change_request_url()
+            self._online_ide = True
+            self._login(online_ide=True)
+            self._logged_in = True
         else:
             self.headers = {
                 "x-csrftoken": "a",
@@ -59,18 +70,19 @@ class ScratchConnect(UserCommon):
                           + ";",
                 "referer": "https://scratch.mit.edu",
             }
-            Warnings.LoginWarning(
-                'Login with Username/Password and Cookie Failed! Continuing without login...')
+            Warnings.warn(
+                "[1m[33mScratchConnect: [31mLogin with Username/Password and Cookie Failed! Continuing without login...[0m")
         super().__init__(self.username,
-                         self.headers)  # Get other properties and methods from the parent(UserCommon) class
+                         self.headers,
+                         self._online_ide)  # Get other properties and methods from the parent(UserCommon) class
         self.update_data()
 
-    def _login(self, cookie=False, auto_cookie_login=False):
+    def _login(self, cookie=False, auto_cookie_login=False, online_ide=False):
         """
         Function to login(don't use this)
         """
         global _user_link
-        if cookie is False:
+        if cookie is False and online_ide is False:
             headers = {
                 "x-csrftoken": "a",
                 "x-requested-with": "XMLHttpRequest",
@@ -87,31 +99,23 @@ class ScratchConnect(UserCommon):
                     if auto_cookie_login is True:
                         self._cookie_login()
                     else:
-                        raise Exceptions.ForbiddenLogin("""
-                         Scratch is not letting you login from this device.
-                         Try to do the following to fix this issue:
-                         - Try again later (10-15 minutes)
-                         - Use Cookie login - https://github.com/Sid72020123/scratchconnect#Cookie-Login
-                         - Try from another device (Scratch sometimes blocks login from Replit)
-                         """)
+                        Warnings.warn(
+                            """[33m[1mScratchConnect: [31mScratch is not letting you login from this device.\n[35mTry to do the following to fix this issue:\n[36m- Try again later (10-15 minutes)\n[36m- Use Cookie login - [4mhttps://github.com/Sid72020123/scratchconnect#Cookie-Login[0m\n[36m[1m- Try from another device (Scratch sometimes blocks login from Replit)[0m""")
             try:
                 self.session_id = re.search('"(.*)"', request.headers["Set-Cookie"]).group()
-                self.token = request.json()[0]["token"]
+                self._get_token()
             except AttributeError:
                 if auto_cookie_login is True:
                     self._cookie_login()
                 else:
                     raise Exceptions.InvalidInfo('Invalid Username or Password!')
-            headers = {
-                "x-requested-with": "XMLHttpRequest",
-                "Cookie": "scratchlanguage=en;permissions=%7B%7D;",
-                "referer": "https://scratch.mit.edu",
-            }
-            request = requests.get("https://scratch.mit.edu/csrf_token/", headers=headers)
-            self.csrf_token = re.search("scratchcsrftoken=(.*?);", request.headers["Set-Cookie"]).group(1)
+            self._get_csrf_token()
             _user_link = f"https://{_api}/users/{self.username}/"
-        else:
+        elif cookie is not False:
             self._cookie_login()
+        elif online_ide is not False:
+            self._online_ide_login()
+
         self.headers = {
             "x-csrftoken": self.csrf_token,
             "X-Token": self.token,
@@ -124,16 +128,65 @@ class ScratchConnect(UserCommon):
             "referer": "https://scratch.mit.edu",
         }
 
+    def _get_csrf_token(self):
+        headers = {
+            "x-requested-with": "XMLHttpRequest",
+            "Cookie": "scratchlanguage=en;permissions=%7B%7D;",
+            "referer": "https://scratch.mit.edu",
+        }
+        request = requests.get("https://scratch.mit.edu/csrf_token/", headers=headers)
+        self.csrf_token = re.search("scratchcsrftoken=(.*?);", request.headers["Set-Cookie"]).group(1)
+
+    def _get_token(self):
+        response = requests.post("https://scratch.mit.edu/session", headers={
+            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.101 Safari/537.36',
+            "x-csrftoken": "a",
+            "x-requested-with": "XMLHttpRequest",
+            "referer": "https://scratch.mit.edu",
+        }, cookies={"scratchsessionsid": self.session_id, "scratchcsrftoken": "a", "scratchlanguage": "en"}).json()
+        self.token = response['user']['token']
+        self.session = response
+        if self.session["user"]["banned"]:
+            raise Exceptions.UnauthorizedAction(
+                "You are banned on Scratch! You cannot login from ScratchConnect unless you are unbanned! This error is raised because ScratchConnect won't allow the banned users to login and do something inappropriate!")
+
     def _cookie_login(self):
+        if self.cookie is None:
+            raise Exceptions.InvalidInfo("Cookie Not Provided!")
         try:
             self.username = self.cookie["Username"]
-            self.token = ""
-            self.csrf_token = self.cookie["CSRFToken"]
             self.session_id = self.cookie["SessionID"]
-            Warnings.CookieLoginWarning(
-                'You are logging in with cookie. Some features might not work if the cookie values are wrong!')
         except KeyError:
             raise Exceptions.InvalidInfo("Required Cookie Headers are missing!")
+        try:
+            self._get_token()
+            self._get_csrf_token()
+            Warnings.warn(
+                "[1m[33mScratchConnect: [31mYou are logging in with cookie. Some features might not work if the cookie values are wrong![0m")
+        except KeyError:
+            Warnings.warn(
+                "[1m[33mScratchConnect: [31mCookie Login Failed because the cookie values may be wrong![0m")
+            self.csrf_token = ""
+            self.token = ""
+
+    def _online_ide_login(self):
+        if self.online_ide_cookie is None:
+            raise Exceptions.InvalidInfo("Cookie Info Not Provided!")
+        try:
+            self.username = self.online_ide_cookie["Username"]
+            self.session_id = self.online_ide_cookie["SessionID"]
+        except KeyError:
+            raise Exceptions.InvalidInfo("Required Cookie Headers are missing!")
+        try:
+            self._get_token()
+            self._get_csrf_token()
+            Warnings.warn(
+                "[1m[33mScratchConnect: [31mYou are logging in on Replit or some other online IDE. Some features might not work if the cookie values are wrong or it may be slow! Also, you can't do any social interactions![0m")
+        except (KeyError, TypeError, ValueError):
+            Warnings.warn(
+                "[1m[33mScratchConnect: [31mFetching token or csrf_token failed! You can still continue but social interactions won't work![0m")
+            self.csrf_token = ""
+            self.token = ""
 
     def check(self, username):
         try:
@@ -413,7 +466,7 @@ class ScratchConnect(UserCommon):
         :param username: A valid Username
         """
         return User.User(username=username, client_username=self.username, headers=self.headers,
-                         logged_in=self._logged_in)
+                         logged_in=self._logged_in, online_ide=self._online_ide)
 
     def connect_studio(self, studio_id):
         """
@@ -421,7 +474,7 @@ class ScratchConnect(UserCommon):
         :param studio_id: A valid studio ID
         """
         return Studio.Studio(id=studio_id, client_username=self.username, headers=self.headers,
-                             logged_in=self._logged_in)
+                             logged_in=self._logged_in, online_ide=self._online_ide)
 
     def connect_project(self, project_id, access_unshared=False):
         """
@@ -430,14 +483,16 @@ class ScratchConnect(UserCommon):
         :param access_unshared: Set to True if you want to connect an unshared project
         """
         return Project.Project(id=project_id, client_username=self.username, headers=self.headers,
-                               logged_in=self._logged_in, unshared=access_unshared, session_id=self.session_id)
+                               logged_in=self._logged_in, unshared=access_unshared, session_id=self.session_id,
+                               online_ide=self._online_ide)
 
     def connect_forum_topic(self, forum_id):
         """
         Connect a Scratch Forum Topic
         :param forum_id: A valid forum topic ID
         """
-        return Forum.Forum(id=forum_id, client_username=self.username, headers=self.headers, logged_in=self._logged_in)
+        return Forum.Forum(id=forum_id, client_username=self.username, headers=self.headers, logged_in=self._logged_in,
+                           online_ide=self._online_ide)
 
     def create_new_terminal(self):
         """
@@ -450,3 +505,9 @@ class ScratchConnect(UserCommon):
         Create a new Chart object
         """
         return _chart(sc=self)
+
+    def create_new_image(self):
+        """
+        Create a new scImage object
+        """
+        return Image(online_ide=self._online_ide)
