@@ -12,8 +12,10 @@ headers = {"Library": "ScratchConnect.py"}
 
 def _get(url, params=None, **kwargs):
     if urlparse(url).netloc in ["api.scratch.mit.edu", "scratch.mit.edu", "cdn2.scratch.mit.edu"]:
-        url = "https://sc-proxy.scratchconnect.eu.org/get/" + url
-    return request("get", url, headers=headers, params=params, **kwargs)
+        url = "https://sc-proxy.deta.dev/get/" + url
+    
+    kwargs["headers"] = headers
+    return request("get", url, params=params, **kwargs)
 
 
 def _change_request_url():
